@@ -13,7 +13,7 @@ def scalar_pars():
     scalar_pars = {
         # Basic parameters
         'location'               : 'senegal',
-        'n_agents'               : 10_000, # Number of agents
+        'n_agents'               : 1_000, # Number of agents
         'scaled_pop'             : None, # Scaled population / total population size
         'start_year'             : 1960, # Start year of simulation
         'end_year'               : 2020, # End year of simulation
@@ -22,6 +22,7 @@ def scalar_pars():
         'seed'                   : 1, # Random seed
         'verbose'                : 1, # How much detail to print during the simulation
         'track_switching'        : 0, # Whether to track method switching
+        'track_as'               : 0, # Whether to track age-specific channels
 
         # Age limits (in years)
         'method_age'             : 15,
@@ -36,6 +37,7 @@ def scalar_pars():
         'postpartum_dur'         : 35,
         'breastfeeding_dur_mu'   : 19.66828, # For a Gumbel distribution, drawn from DHS
         'breastfeeding_dur_beta' : 7.2585,
+        'max_lam_dur'            : 5, # Duration of lactational amenorrhea
 
         # Pregnancy outcomes
         'abortion_prob'             : 0.08, # From https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4712915/
@@ -73,11 +75,12 @@ def filenames():
     files = {}
     files['base'] = sc.thisdir(aspath=True) / 'senegal'
     files['basic_dhs']        = 'senegal-basic-dhs.yaml'
-    files['pregnancy_parity'] = 'SNIR80FL.obj'
     files['popsize']          = 'senegal-popsize.csv'
-    files['mcpr']             = 'mcpr_senegal.csv'
+    files['mcpr']             = 'senegal-mcpr.csv'
     files['tfr']              = 'senegal-tfr.csv'
+    files['asfr']             = 'senegal-asfr.csv'
     files['skyscrapers']      = 'Skyscrapers-All-DHS.csv'
+    files['pregnancy_parity'] = 'SNIR80FL.obj'
     files['spacing']          = 'BirthSpacing.obj'
     files['methods']          = 'Method_v312.csv'
     return files
@@ -276,6 +279,8 @@ def stillbirth():
     '''
     From Report of the UN Inter-agency Group for Child Mortality Estimation, 2020
     https://childmortality.org/wp-content/uploads/2020/10/UN-IGME-2020-Stillbirth-Report.pdf
+    
+    Age adjustments come from an extension of Noori et al., which were conducted June 2022.
     '''
 
     data = np.array([
@@ -287,6 +292,8 @@ def stillbirth():
     stillbirth_rate = {}
     stillbirth_rate['year'] = data[:,0]
     stillbirth_rate['probs'] = data[:,1]/1000    # Rate per 1000 total births
+    stillbirth_rate['ages']      = np.array([15,   16,   17,   19,   20,   28,  31,  36,   50])
+    stillbirth_rate['age_probs'] = np.array([3.27, 1.64, 1.85, 1.39, 0.89, 1.0, 1.5, 1.55, 1.78]) #odds ratios 
 
     return stillbirth_rate
 
@@ -551,143 +558,39 @@ def birth_spacing_pref():
 
 def methods():
     '''
-    Matrices to give transitional probabilities from 2018 DHS Senegal contraceptive calendar data
-    Probabilities in this function are annual probabilities of initiating, discontinuing, continuing
-    or switching methods.
-
-    Probabilities at postpartum month 1 are 1 month transitional probabilities for starting a method after delivery.
-
-    Probabilities at postpartum month 6 are 5 month transitional probabilities for starting or changing methods over
-    the first 6 months postpartum.
-
-    Data from Senegal DHS contraceptive calendars, 2017 and 2018 combined
+    Names, indices, modern/traditional flag, and efficacies of contraceptive methods -- see also parameters.py
+    Efficacy from Guttmacher, fp_prerelease/docs/gates_review/contraceptive-failure-rates-in-developing-world_1.pdf
+    BTL failure rate from general published data
+    Pooled efficacy rates for all women in this study: https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4970461/
     '''
-    methods = {}
 
-    # Names and indices of contraceptive methods -- see also defaults.py
-    methods['map'] = {
-        'None'              : 0,
-        'Pill'              : 1,
-        'IUDs'              : 2,
-        'Injectables'       : 3,
-        'Condoms'           : 4,
-        'BTL'               : 5,
-        'Withdrawal'        : 6,
-        'Implants'          : 7,
-        'Other traditional' : 8,
-        'Other modern'      : 9,
+    # Define method data
+    data = {                # Index, modern, efficacy
+        'None'              : [0, False, 0.000],
+        'Withdrawal'        : [1, False, 0.866],
+        'Other traditional' : [2, False, 0.861], # 1/2 periodic abstinence, 1/2 other traditional approx.  Using rate from periodic abstinence
+        'Condoms'           : [3, True,  0.946],
+        'Pill'              : [4, True,  0.945],
+        'Injectables'       : [5, True,  0.983],
+        'Implants'          : [6, True,  0.994],
+        'IUDs'              : [7, True,  0.986],
+        'BTL'               : [8, True,  0.995],
+        'Other modern'      : [9, True,  0.880], # SDM makes up about 1/2 of this, perfect use is 95% and typical is 88%.  EC also included here, efficacy around 85% https : //www.aafp.org/afp/2004/0815/p707.html
     }
 
+    keys = data.keys()
+    methods = {}
+    methods['map']    = {k:data[k][0] for k in keys}
+    methods['modern'] = {k:data[k][1] for k in keys}
+    methods['eff']    = {k:data[k][2] for k in keys}
+
     # Age bins for different method switching matrices -- duplicated in defaults.py
     methods['age_map'] = {
-        '<18':   [ 0, 18],
-        '18-20': [18, 20],
-        '21-25': [20, 25],
-        '>25':   [25, fpd.max_age+1], # +1 since we're using < rather than <=
-    }
-
-    methods['raw'] = {}
-    methods['raw']['annual'] = {
-        '<18': np.array([
-            [0.9953132643, 0.0001774295, 0.0000506971, 0.0016717604, 0.0011907588, 0.0000000000, 0.0000000000, 0.0013933117, 0.0001774295, 0.0000253488],
-            [0.5357744265, 0.3956817610, 0.0000000000, 0.0685438124, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.3779420208, 0.0357636560, 0.0000000000, 0.5646930063, 0.0000000000, 0.0000000000, 0.0000000000, 0.0216013169, 0.0000000000, 0.0000000000],
-            [0.3069791534, 0.0000000000, 0.0000000000, 0.0000000000, 0.6930208466, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.2002612324, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.7997387676, 0.0000000000, 0.0000000000],
-            [0.0525041055, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.9474958945, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000]]),
-        '18-20': np.array([
-            [0.9773550478, 0.0027107826, 0.0002856633, 0.0103783776, 0.0027107826, 0.0000000000, 0.0000000000, 0.0048461143, 0.0014275685, 0.0002856633],
-            [0.4549090811, 0.4753537374, 0.0000000000, 0.0463236223, 0.0000000000, 0.0000000000, 0.0000000000, 0.0234135593, 0.0000000000, 0.0000000000],
-            [0.1606987622, 0.0000000000, 0.8393012378, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.4388765348, 0.0196152254, 0.0000000000, 0.5217816111, 0.0098521372, 0.0000000000, 0.0000000000, 0.0049372457, 0.0049372457, 0.0000000000],
-            [0.1820343429, 0.0000000000, 0.0000000000, 0.0000000000, 0.8179656571, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.1700250992, 0.0000000000, 0.0000000000, 0.0196339461, 0.0000000000, 0.0000000000, 0.0000000000, 0.8103409547, 0.0000000000, 0.0000000000],
-            [0.3216043236, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.6783956764, 0.0000000000],
-            [0.4773308708, 0.0000000000, 0.0000000000, 0.0000000000, 0.4773308708, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0453382584]]),
-        '21-25': np.array([
-            [0.9580844982, 0.0080857514, 0.0005744238, 0.0183734782, 0.0024392127, 0.0000000000, 0.0001436343, 0.0107913587, 0.0010767964, 0.0004308462],
-            [0.3714750489, 0.5703031529, 0.0000000000, 0.0434765586, 0.0000000000, 0.0000000000, 0.0029538440, 0.0088375516, 0.0029538440, 0.0000000000],
-            [0.1079007053, 0.0445194312, 0.8250878450, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0224920185, 0.0000000000, 0.0000000000],
-            [0.3776516335, 0.0257873675, 0.0023699011, 0.5835323377, 0.0035529200, 0.0000000000, 0.0000000000, 0.0035529200, 0.0035529200, 0.0000000000],
-            [0.1895749834, 0.0094301857, 0.0000000000, 0.0000000000, 0.7540070663, 0.0000000000, 0.0000000000, 0.0187787895, 0.0094301857, 0.0187787895],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.4471746024, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.5528253976, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.1370415572, 0.0044750998, 0.0029854437, 0.0044750998, 0.0029854437, 0.0000000000, 0.0000000000, 0.8480373557, 0.0000000000, 0.0000000000],
-            [0.2375959813, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.7624040187, 0.0000000000],
-            [0.3342350643, 0.0000000000, 0.0000000000, 0.0000000000, 0.1826479897, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.4831169460]]),
-        '>25': np.array([
-            [0.9462131116, 0.0123838433, 0.0029627055, 0.0209275401, 0.0013426008, 0.0001399314, 0.0001399314, 0.0139046297, 0.0017618246, 0.0002238816],
-            [0.3030708445, 0.6588856456, 0.0053038550, 0.0210610879, 0.0021246451, 0.0000000000, 0.0015938721, 0.0053038550, 0.0021246451, 0.0005315497],
-            [0.0774565806, 0.0058280554, 0.9050475430, 0.0043739679, 0.0000000000, 0.0000000000, 0.0000000000, 0.0043739679, 0.0014599426, 0.0014599426],
-            [0.2745544442, 0.0173002840, 0.0047711954, 0.6910279110, 0.0019671434, 0.0000000000, 0.0000000000, 0.0072886657, 0.0028091184, 0.0002812379],
-            [0.1548546967, 0.0041688319, 0.0083217274, 0.0124587419, 0.8077054428, 0.0000000000, 0.0000000000, 0.0083217274, 0.0000000000, 0.0041688319],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0939227324, 0.0000000000, 0.0000000000, 0.0480226276, 0.0000000000, 0.0000000000, 0.8580546400, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.1114787085, 0.0059115428, 0.0024929872, 0.0068420105, 0.0003119354, 0.0003119354, 0.0003119354, 0.8714034061, 0.0009355387, 0.0000000000],
-            [0.1060688610, 0.0025335150, 0.0025335150, 0.0025335150, 0.0050611449, 0.0000000000, 0.0000000000, 0.0050611449, 0.8762083044, 0.0000000000],
-            [0.1581337279, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0121394565, 0.8297268156]])
-    }
-
-    # Postpartum probabilities, 0-1 month
-    methods['raw']['pp0to1'] = {
-        '<18':   np.array([0.9606501283, 0.0021385800, 0.0004277160, 0.0128314799, 0.0008554320, 0.0000000000, 0.0008554320, 0.0205303678, 0.0017108640, 0.0000000000]),
-        '18-20': np.array([0.9524886878, 0.0028280543, 0.0005656109, 0.0214932127, 0.0005656109, 0.0000000000, 0.0005656109, 0.0197963801, 0.0016968326, 0.0000000000]),
-        '21-25': np.array([0.9379245283, 0.0083018868, 0.0013207547, 0.0284905660, 0.0009433962, 0.0000000000, 0.0000000000, 0.0177358491, 0.0052830189, 0.0000000000]),
-        '>25':   np.array([0.9253704535, 0.0102379883, 0.0040413112, 0.0264930400, 0.0007184553, 0.0022451729, 0.0001796138, 0.0267624607, 0.0035922766, 0.0003592277])
-    }
-
-    # Postpartum probabilities, 1-6 months
-    methods['raw']['pp1to6'] = {
-        '<18': np.array([
-            [0.9013605442, 0.0126336249, 0.0004859086, 0.0510204082, 0.0009718173, 0.0000000000, 0.0000000000, 0.0272108844, 0.0063168124, 0.0000000000],
-            [0.4000000000, 0.6000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0714285714, 0.0000000000, 0.0000000000, 0.9285714286, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.5000000000, 0.5000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000]]),
-        '18-20': np.array([
-            [0.8774703557, 0.0191040843, 0.0046113307, 0.0586297760, 0.0032938076, 0.0000000000, 0.0006587615, 0.0329380764, 0.0026350461, 0.0006587615],
-            [0.0000000000, 0.7500000000, 0.0000000000, 0.2500000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0277777778, 0.0000000000, 0.0000000000, 0.9722222222, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0312500000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.9687500000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000]]),
-        '21-25': np.array([
-            [0.8538140251, 0.0279182238, 0.0021982853, 0.0721037591, 0.0037370851, 0.0000000000, 0.0004396571, 0.0342932513, 0.0054957133, 0.0000000000],
-            [0.0243902439, 0.9512195122, 0.0000000000, 0.0243902439, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0671641791, 0.0000000000, 0.0000000000, 0.9328358209, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.2500000000, 0.0000000000, 0.2500000000, 0.5000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0246913580, 0.0000000000, 0.0000000000, 0.0123456790, 0.0000000000, 0.0000000000, 0.0000000000, 0.9629629630, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0416666667, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.9583333333, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000]]),
-        '>25': np.array([
-            [0.8433467315, 0.0289824413, 0.0070869473, 0.0691770679, 0.0040194627, 0.0001057753, 0.0008462027, 0.0390310979, 0.0064522953, 0.0009519780],
-            [0.0673076923, 0.8653846154, 0.0000000000, 0.0288461538, 0.0000000000, 0.0000000000, 0.0000000000, 0.0384615385, 0.0000000000, 0.0000000000],
-            [0.0256410256, 0.0000000000, 0.9487179487, 0.0256410256, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0272373541, 0.0077821012, 0.0000000000, 0.9533073930, 0.0000000000, 0.0000000000, 0.0000000000, 0.0077821012, 0.0038910506, 0.0000000000],
-            [0.1666666667, 0.0000000000, 0.0000000000, 0.0000000000, 0.6666666667, 0.0000000000, 0.0000000000, 0.1666666667, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.5000000000, 0.5000000000, 0.0000000000, 0.0000000000],
-            [0.0109090909, 0.0036363636, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.9854545455, 0.0000000000, 0.0000000000],
-            [0.0270270270, 0.0000000000, 0.0000000000, 0.0270270270, 0.0000000000, 0.0000000000, 0.0000000000, 0.0270270270, 0.9189189189, 0.0000000000],
-            [0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 0.0000000000, 1.0000000000]])
+        '<18'   : [ 0, 18],
+        '18-20' : [18, 20],
+        '21-25' : [20, 25],
+        '26-35' : [25, 35],
+        '>35'   : [35, fpd.max_age+1], # +1 since we're using < rather than <=
     }
 
     # Data on trend in MCPR in Senegal over time, in % # CK: TODO: find source
@@ -697,59 +600,153 @@ def methods():
     return methods
 
 
-def efficacy(disable=False):
+def method_probs():
     '''
-    From Guttmacher, fp/docs/gates_review/contraceptive-failure-rates-in-developing-world_1.pdf
-    BTL failure rate from general published data
-    Pooled efficacy rates for all women in this study: https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4970461/
-    '''
-    method_efficacy = sc.odict({
-        'None'              :  0.0,
-        'Pill'              : 94.5,
-        'IUDs'              : 98.6,
-        'Injectable'        : 98.3,
-        'Condoms'           : 94.6,
-        'BTL'               : 99.5,
-        'Withdrawal'        : 86.6,
-        'Implants'          : 99.4,
-        'Other traditional' : 86.1, # 1/2 periodic abstinence, 1/2 other traditional approx.  Using rate from periodic abstinence
-        'Other modern'      : 88.0, # SDM makes up about 1/2 of this, perfect use is 95% and typical is 88%.  EC also included here, efficacy around 85% https : //www.aafp.org/afp/2004/0815/p707.html
-    })
+    Define "raw" (un-normalized, un-trended) matrices to give transitional probabilities
+    from 2018 DHS Senegal contraceptive calendar data.
 
-    if disable:
-        method_efficacy[:] = 100 # To disable contraception
+    Probabilities in this function are annual probabilities of initiating (top row), discontinuing (first column),
+    continuing (diagonal), or switching methods (all other entries).
 
-    method_efficacy = method_efficacy[:]/100
+    Probabilities at postpartum month 1 are 1 month transitional probabilities
+    for starting a method after delivery.
 
-    return method_efficacy
+    Probabilities at postpartum month 6 are 5 month transitional probabilities
+    for starting or changing methods over the first 6 months postpartum.
 
-
-def efficacy25(disable=False):
-    '''
-    From Guttmacher, fp/docs/gates_review/contraceptive-failure-rates-in-developing-world_1.pdf
-    BTL failure rate from general published data
-    Pooled efficacy rates for women ages 25+
+    Data from Senegal DHS contraceptive calendars, 2017 and 2018 combined
     '''
 
-    method_efficacy25 = sc.odict({
-        'None'       :  0.0,
-        'Pill'       : 91.7,
-        'IUDs'       : 96.8,
-        'Injectable' : 96.5,
-        'Condoms'    : 91.1,
-        'BTL'        : 99.5,
-        'Rhythm'     : 75.4,
-        'Withdrawal' : 77.3,
-        'Implants'   : 99.4,
-        'Other'      : 94.5,
-    })
+    raw = {
 
-    if disable:
-        method_efficacy25[:] = 100 # To disable contraception
+    # Main switching matrix: all non-postpartum women
+    'annual': {
+        '<18': np.array([
+           [0.9953, 0.    , 0.0002, 0.0012, 0.0002, 0.0017, 0.0014, 0.0001, 0.    , 0.    ],
+           [0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.0525, 0.    , 0.9475, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.307 , 0.    , 0.    , 0.693 , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.5358, 0.    , 0.    , 0.    , 0.3957, 0.0685, 0.    , 0.    , 0.    , 0.    ],
+           [0.3779, 0.    , 0.    , 0.    , 0.0358, 0.5647, 0.0216, 0.    , 0.    , 0.    ],
+           [0.2003, 0.    , 0.    , 0.    , 0.    , 0.    , 0.7997, 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000]]),
+        '18-20': np.array([
+           [0.9774, 0.    , 0.0014, 0.0027, 0.0027, 0.0104, 0.0048, 0.0003, 0.    , 0.0003],
+           [0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.3216, 0.    , 0.6784, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.182 , 0.    , 0.    , 0.818 , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.4549, 0.    , 0.    , 0.    , 0.4754, 0.0463, 0.0234, 0.    , 0.    , 0.    ],
+           [0.4389, 0.    , 0.0049, 0.0099, 0.0196, 0.5218, 0.0049, 0.    , 0.    , 0.    ],
+           [0.17  , 0.    , 0.    , 0.    , 0.    , 0.0196, 0.8103, 0.    , 0.    , 0.    ],
+           [0.1607, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.8393, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.4773, 0.    , 0.    , 0.4773, 0.    , 0.    , 0.    , 0.    , 0.    , 0.0453]]),
+        '21-25': np.array([
+           [0.9581, 0.0001, 0.0011, 0.0024, 0.0081, 0.0184, 0.0108, 0.0006, 0.    , 0.0004],
+           [0.4472, 0.5528, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.2376, 0.    , 0.7624, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.1896, 0.    , 0.0094, 0.754 , 0.0094, 0.    , 0.0188, 0.    , 0.    , 0.0188],
+           [0.3715, 0.003 , 0.003 , 0.    , 0.5703, 0.0435, 0.0088, 0.    , 0.    , 0.    ],
+           [0.3777, 0.    , 0.0036, 0.0036, 0.0258, 0.5835, 0.0036, 0.0024, 0.    , 0.    ],
+           [0.137 , 0.    , 0.    , 0.003 , 0.0045, 0.0045, 0.848 , 0.003 , 0.    , 0.    ],
+           [0.1079, 0.    , 0.    , 0.    , 0.0445, 0.    , 0.0225, 0.8251, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.3342, 0.    , 0.    , 0.1826, 0.    , 0.    , 0.    , 0.    , 0.    , 0.4831]]),
+        '26-35': np.array([
+           [0.9462, 0.0001, 0.0018, 0.0013, 0.0124, 0.0209, 0.0139, 0.003 , 0.0001, 0.0002],
+           [0.0939, 0.8581, 0.    , 0.    , 0.    , 0.048 , 0.    , 0.    , 0.    , 0.    ],
+           [0.1061, 0.    , 0.8762, 0.0051, 0.0025, 0.0025, 0.0051, 0.0025, 0.    , 0.    ],
+           [0.1549, 0.    , 0.    , 0.8077, 0.0042, 0.0125, 0.0083, 0.0083, 0.    , 0.0042],
+           [0.3031, 0.0016, 0.0021, 0.0021, 0.6589, 0.0211, 0.0053, 0.0053, 0.    , 0.0005],
+           [0.2746, 0.    , 0.0028, 0.002 , 0.0173, 0.691 , 0.0073, 0.0048, 0.    , 0.0003],
+           [0.1115, 0.0003, 0.0009, 0.0003, 0.0059, 0.0068, 0.8714, 0.0025, 0.0003, 0.    ],
+           [0.0775, 0.    , 0.0015, 0.    , 0.0058, 0.0044, 0.0044, 0.905 , 0.    , 0.0015],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.1581, 0.    , 0.0121, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.8297]]),
+        '>35': np.array([
+           [0.9462, 0.0001, 0.0018, 0.0013, 0.0124, 0.0209, 0.0139, 0.003 , 0.0001, 0.0002],
+           [0.0939, 0.8581, 0.    , 0.    , 0.    , 0.048 , 0.    , 0.    , 0.    , 0.    ],
+           [0.1061, 0.    , 0.8762, 0.0051, 0.0025, 0.0025, 0.0051, 0.0025, 0.    , 0.    ],
+           [0.1549, 0.    , 0.    , 0.8077, 0.0042, 0.0125, 0.0083, 0.0083, 0.    , 0.0042],
+           [0.3031, 0.0016, 0.0021, 0.0021, 0.6589, 0.0211, 0.0053, 0.0053, 0.    , 0.0005],
+           [0.2746, 0.    , 0.0028, 0.002 , 0.0173, 0.691 , 0.0073, 0.0048, 0.    , 0.0003],
+           [0.1115, 0.0003, 0.0009, 0.0003, 0.0059, 0.0068, 0.8714, 0.0025, 0.0003, 0.    ],
+           [0.0775, 0.    , 0.0015, 0.    , 0.0058, 0.0044, 0.0044, 0.905 , 0.    , 0.0015],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.1581, 0.    , 0.0121, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.8297]])
+        },
 
-    method_efficacy25 = method_efficacy25[:]/100
+    # Postpartum switching matrix, 1 to 6 months
+    'pp1to6': {
+        '<18': np.array([
+           [0.9014, 0.    , 0.0063, 0.001 , 0.0126, 0.051 , 0.0272, 0.0005, 0.    , 0.    ],
+           [0.    , 0.5   , 0.    , 0.    , 0.    , 0.    , 0.5   , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.4   , 0.    , 0.    , 0.    , 0.6   , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.0714, 0.    , 0.    , 0.    , 0.    , 0.9286, 0.    , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000]]),
+        '18-20': np.array([
+           [0.8775, 0.0007, 0.0026, 0.0033, 0.0191, 0.0586, 0.0329, 0.0046, 0.    , 0.0007],
+           [0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.75  , 0.25  , 0.    , 0.    , 0.    , 0.    ],
+           [0.0278, 0.    , 0.    , 0.    , 0.    , 0.9722, 0.    , 0.    , 0.    , 0.    ],
+           [0.0312, 0.    , 0.    , 0.    , 0.    , 0.    , 0.9688, 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000]]),
+        '21-25': np.array([
+           [0.8538, 0.0004, 0.0055, 0.0037, 0.0279, 0.0721, 0.0343, 0.0022, 0.    , 0.    ],
+           [0.    , 1.0000, 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.9583, 0.    , 0.    , 0.0417, 0.    , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.5   , 0.25  , 0.25  , 0.    , 0.    , 0.    , 0.    ],
+           [0.0244, 0.    , 0.    , 0.    , 0.9512, 0.0244, 0.    , 0.    , 0.    , 0.    ],
+           [0.0672, 0.    , 0.    , 0.    , 0.    , 0.9328, 0.    , 0.    , 0.    , 0.    ],
+           [0.0247, 0.    , 0.    , 0.    , 0.    , 0.0123, 0.963 , 0.    , 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000]]),
+        '26-35': np.array([
+           [0.8433, 0.0008, 0.0065, 0.004 , 0.029 , 0.0692, 0.039 , 0.0071, 0.0001, 0.001 ],
+           [0.    , 0.5   , 0.    , 0.    , 0.    , 0.    , 0.5   , 0.    , 0.    , 0.    ],
+           [0.027 , 0.    , 0.9189, 0.    , 0.    , 0.027 , 0.027 , 0.    , 0.    , 0.    ],
+           [0.1667, 0.    , 0.    , 0.6667, 0.    , 0.    , 0.1667, 0.    , 0.    , 0.    ],
+           [0.0673, 0.    , 0.    , 0.    , 0.8654, 0.0288, 0.0385, 0.    , 0.    , 0.    ],
+           [0.0272, 0.    , 0.0039, 0.    , 0.0078, 0.9533, 0.0078, 0.    , 0.    , 0.    ],
+           [0.0109, 0.    , 0.    , 0.    , 0.0036, 0.    , 0.9855, 0.    , 0.    , 0.    ],
+           [0.0256, 0.    , 0.    , 0.    , 0.    , 0.0256, 0.    , 0.9487, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000]]),
+        '>35': np.array([
+           [0.8433, 0.0008, 0.0065, 0.004 , 0.029 , 0.0692, 0.039 , 0.0071, 0.0001, 0.001 ],
+           [0.    , 0.5   , 0.    , 0.    , 0.    , 0.    , 0.5   , 0.    , 0.    , 0.    ],
+           [0.027 , 0.    , 0.9189, 0.    , 0.    , 0.027 , 0.027 , 0.    , 0.    , 0.    ],
+           [0.1667, 0.    , 0.    , 0.6667, 0.    , 0.    , 0.1667, 0.    , 0.    , 0.    ],
+           [0.0673, 0.    , 0.    , 0.    , 0.8654, 0.0288, 0.0385, 0.    , 0.    , 0.    ],
+           [0.0272, 0.    , 0.0039, 0.    , 0.0078, 0.9533, 0.0078, 0.    , 0.    , 0.    ],
+           [0.0109, 0.    , 0.    , 0.    , 0.0036, 0.    , 0.9855, 0.    , 0.    , 0.    ],
+           [0.0256, 0.    , 0.    , 0.    , 0.    , 0.0256, 0.    , 0.9487, 0.    , 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000, 0.    ],
+           [0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 1.0000]])
+        },
 
-    return method_efficacy25
+    # Postpartum initiation vectors, 0 to 1 month
+    'pp0to1': {
+        '<18':   np.array([0.9607, 0.0009, 0.0017, 0.0009, 0.0021, 0.0128, 0.0205, 0.0004, 0.    , 0.    ]),
+        '18-20': np.array([0.9525, 0.0006, 0.0017, 0.0006, 0.0028, 0.0215, 0.0198, 0.0006, 0.    , 0.    ]),
+        '21-25': np.array([0.9379, 0.    , 0.0053, 0.0009, 0.0083, 0.0285, 0.0177, 0.0013, 0.    , 0.    ]),
+        '26-35': np.array([0.9254, 0.0002, 0.0036, 0.0007, 0.0102, 0.0265, 0.0268, 0.004 , 0.0022, 0.0004]),
+        '>35':   np.array([0.9254, 0.0002, 0.0036, 0.0007, 0.0102, 0.0265, 0.0268, 0.004 , 0.0022, 0.0004]),
+        }
+    }
+    return raw
 
 
 def barriers():
@@ -801,40 +798,8 @@ def make_pars():
     pars['spacing_pref']       = birth_spacing_pref()
 
     # Contraceptive methods
-    pars['method_efficacy']   = efficacy()
-    pars['method_efficacy25'] = efficacy25()
-    pars['methods']           = methods()
-    pars['barriers']          = barriers()
+    pars['methods']        = methods()
+    pars['methods']['raw'] = method_probs()
+    pars['barriers']       = barriers()
 
-    # Perform validation
-    validate_pars(pars)
-
-    return pars
-
-
-def validate_pars(pars):
-    ''' Perform internal validation checks and other housekeeping '''
-
-    # Validate method matrices
-    method_map = pars['methods']['map']
-    method_age_map = pars['methods']['age_map']
-    n = len(method_map)
-    raw = pars['methods']['raw']
-    age_keys = set(method_age_map.keys())
-    for mkey in ['annual', 'pp0to1', 'pp1to6']:
-        m_age_keys = set(raw[mkey].keys())
-        assert age_keys == m_age_keys, f'Matrix "{mkey}" has inconsistent keys: "{sc.strjoin(age_keys)}" ≠ "{sc.strjoin(m_age_keys)}"'
-    for k in age_keys:
-        shape = raw['pp0to1'][k].shape
-        assert shape == (n,), f'Postpartum method initiation matrix for ages {k} has unexpected shape: should be ({n},), not {shape}'
-        for mkey in ['annual', 'pp1to6']:
-            shape = raw[mkey][k].shape
-            assert shape == (n,n), f'Method matrix {mkey} for ages {k} has unexpected shape: should be ({n},{n}), not {shape}'
-
-    # Copy to defaults, preserving original object ID
-    for k,v in method_map.items():
-        fpd.method_map[k] = v
-    for k,v in method_age_map.items():
-        fpd.method_age_map[k] = v
-
-    return
\ No newline at end of file
+    return pars
\ No newline at end of file
